#include <d3dx9.h>
#include <fstream>
#include <string>
#include <iostream>

#include "resource.h"

using namespace std;

#define LU_LIMIT	10	// How many lines that need to be cleared to level up

#define PixelPositionX(x)	(float)(x)*32+48
#define PixelPositionY(x)	608.0f-((float)(x)*32)
#define GridPositionX(x)	((int)(x)-48)/32
#define GridPositionY(x)	(int)(608.0f-(x))/32

//------------------------------------------------------------------------------------------
// GLOBALS
//------------------------------------------------------------------------------------------
LPDIRECT3D9					g_pD3D			  = NULL;	// Object used to create the device
LPDIRECT3DDEVICE9			g_pd3dDevice	  = NULL;	// Object used in rendering

LPDIRECT3DTEXTURE9			mainScreenTex	  = NULL;	// Menu Screen Texture
LPDIRECT3DTEXTURE9			helpScreenTex	  = NULL;	// Help Screen Texture
LPDIRECT3DTEXTURE9			gameOverScreenTex = NULL;	// Game over Screen Texture
LPDIRECT3DTEXTURE9			bgTexture		  = NULL;	// Background Brick Texture
LPDIRECT3DTEXTURE9			blueBrickTex	  = NULL;	// Blue Brick Texture
LPDIRECT3DTEXTURE9			redBrickTex		  = NULL;	// Red Brick Texture
LPDIRECT3DTEXTURE9			lblueBrickTex	  = NULL;	// Light Blue Brick Texture
LPDIRECT3DTEXTURE9			greenBrickTex	  = NULL;	// Green Brick Texture
LPDIRECT3DTEXTURE9			yellowBrickTex	  = NULL;	// Yellow Brick Texture
LPDIRECT3DTEXTURE9			pinkBrickTex	  = NULL;	// Pink Brick Texture 
LPDIRECT3DTEXTURE9			greyBrickTex	  = NULL;	// Grey Brick Texture 

LPD3DXSPRITE				sprite			  = NULL;	// Background sprite

LPD3DXFONT font								  = NULL;	// Font object

LPDIRECT3DTEXTURE9* color[20][10];						// A grid 20 rows x 10 columns

D3DXVECTOR3 bgPosition( 1.0f, 0.0f, 0.0f );				// Background (border) position

unsigned int begTime = timeGetTime();

struct ScoreStruct
{
	char name[10];
	int score;
} highScore[6];

struct BlockStruct 
{
	D3DXVECTOR3 position;		// Poistion of the block
	LPDIRECT3DTEXTURE9* color;	// Pointer to a texture
} nextBlock[4], currentBlock[4];

// Some colors that the user can switch between for the background color
D3DCOLOR backgroundColor[8] = {	D3DCOLOR_XRGB( 0, 0, 0 ), 
								D3DCOLOR_XRGB( 255, 255, 255 ), 
								D3DCOLOR_XRGB( 0, 0, 150 ), 
								D3DCOLOR_XRGB( 0, 150, 0 ), 
								D3DCOLOR_XRGB( 127, 255, 212 ), 
								D3DCOLOR_XRGB( 244, 164, 96 ), 
								D3DCOLOR_XRGB( 255, 64, 64 ), 
								D3DCOLOR_XRGB( 179, 238, 55 ) }; 

int nameArrayPos = 0;
int charNum = 0;
int colorNum = 0;	// The background color ( chhoses the element of the colors array ) - starts at black
int blockType;		// random number choosing the type of block is created
int speed = 500;	// Speed of the block - starts off at 500
int moveChance = 0;	// Used to give you a chance to move the block once it hits a static block
int numOfRows = 0;
int score = 0;
int level = 1;
int numOfRowsDropped = 0;
int linesCleared = 0;
bool levelUp = false;
enum mainStatus { mainScreen, helpScreen, inGame, paused, gameOver, highScores, getNameForScoreboard }; // enum to hold the game status
mainStatus gameStatus = mainScreen; // Start with the menu on



//----------------------------------------------------------------------------------------------------
// Name: CreateTextures()
// Desc: Creates the textures from the images
//----------------------------------------------------------------------------------------------------
HRESULT CreateTextures()
{
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_HELP_SCREEN, &helpScreenTex ) ) )
	{
		MessageBox( NULL, "Can't find 'helpScreen.jpg', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_MAIN_SCREEN, &mainScreenTex ) ) )
	{
		MessageBox( NULL, "Can't find 'mainScreen.jpg', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_BRICK_EDGE, &bgTexture ) ) )
	{
		MessageBox( NULL, "Can't find 'brickEdge.bmp', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_GAMEOVER_SCREEN, &gameOverScreenTex ) ) )
	{
		MessageBox( NULL, "Can't find 'gameOver.jpg', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_BLUE_BLOCK, &blueBrickTex ) ) )
	{
		MessageBox( NULL, "Can't find 'blueBlock.bmp', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_RED_BLOCK, &redBrickTex ) ) )
	{
		MessageBox( NULL, "Can't find 'redBlock.bmp', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_LIGHTBLUE_BLOCK, &lblueBrickTex ) ) )
	{
		MessageBox( NULL, "Can't find 'lightblueBlock.bmp', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_GREEN_BLOCK, &greenBrickTex ) ) )
	{
		MessageBox( NULL, "Can't find 'greenBlock.bmp', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_YELLOW_BLOCK, &yellowBrickTex ) ) )
	{
		MessageBox( NULL, "Can't find 'yellowBlock.bmp', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_PINK_BLOCK, &pinkBrickTex ) ) )
	{
		MessageBox( NULL, "Can't find 'pinkBlock.bmp', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}
	if ( FAILED ( D3DXCreateTextureFromResource( g_pd3dDevice, NULL, (LPCSTR)ID_GREY_BLOCK, &greyBrickTex ) ) )
	{
		MessageBox( NULL, "Can't find 'greyBlock.bmp', make sure its in the textures folder", "Error!", MB_OK );
		return E_FAIL;
	}

	// Set all the pointers in our array to NULL;
	for ( int y = 0 ; y < 20 ; y++ )
	{
		for ( int x = 0 ; x < 10 ; x++ )
			color[y][x] = NULL;
	}

	srand( timeGetTime() );	 // Seed the randomizer, needed for creation of our blocks

	return S_OK;
}


//--------------------------------------------------------------------------------------------------------------
// Name: UpdateLevel()
// Desc: Changes the level if the appropriate number of lines have been cleared
//--------------------------------------------------------------------------------------------------------------
void UpdateLevel( int &linesCleared, bool &levelUp )
{
	RECT levelUpNotePosition = { 48, 300, 360, 640 };
	if ( linesCleared > LU_LIMIT )
	{
		levelUp = true;
		// Set all the pointers in our array to NULL;
		for ( int y = 0 ; y < 20 ; y++ )
		{
			for ( int x = 0 ; x < 10 ; x++ )
				color[y][x] = NULL;
		}
		linesCleared = 0;
		level++;
		if ( level < 20 )	
			speed -= 20;		
	}
}


//------------------------------------------------------------------------------------------------------------------------------------------------
// Name: ResetVariables()
// Desc: Resets all the variables so we can start a new game
//------------------------------------------------------------------------------------------------------------------------------------------------
void ResetVariables()
{
	linesCleared = 11;						// This is set to anything > 10 so UpdateLevel() clears the grid array
	UpdateLevel( linesCleared, levelUp );	// Run this function so the grid array is cleared
	levelUp = false;						// UpdateLevel() changes this to true, so we want to reset it to false
	level = 0;								// Reset
	score = 0;								// Reset
}

//------------------------------------------------------------------------------------------------------------------------------------------------
// Name: CreateNextBlock()
// Desc: Creates the next block which appears in the top right of the screen
//------------------------------------------------------------------------------------------------------------------------------------------------
void CreateNextBlock( BlockStruct newBrick[4] )
{
	// First transfer contents of nextBlock to currentBlock
	currentBlock[0].position = nextBlock[0].position;
	currentBlock[1].position = nextBlock[1].position;
	currentBlock[2].position = nextBlock[2].position;
	currentBlock[3].position = nextBlock[3].position;
	currentBlock[0].color = nextBlock[0].color;
	currentBlock[1].color = nextBlock[1].color;
	currentBlock[2].color = nextBlock[2].color;
	currentBlock[3].color = nextBlock[3].color;

	blockType = rand()%7+1; // Random number between 1 and 7
	
	switch ( blockType )
	{
	case 1:
		nextBlock[0].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(22), 0.0f );
		nextBlock[1].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(21), 0.0f );
		nextBlock[2].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(20), 0.0f );
		nextBlock[3].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(19), 0.0f );
		nextBlock[0].color = nextBlock[1].color = nextBlock[2].color = nextBlock[3].color = &redBrickTex;
		break;
	case 2:
		nextBlock[0].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(20), 0.0f );
		nextBlock[1].position = D3DXVECTOR3( PixelPositionX(6), PixelPositionY(20), 0.0f );
		nextBlock[2].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(20), 0.0f );
		nextBlock[3].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(19), 0.0f );
		nextBlock[0].color = nextBlock[1].color = nextBlock[2].color = nextBlock[3].color = &greyBrickTex;
		break;
	case 3:
		nextBlock[0].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(19), 0.0f );
		nextBlock[1].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(19), 0.0f );
		nextBlock[2].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(20), 0.0f );
		nextBlock[3].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(20), 0.0f );
		nextBlock[0].color = nextBlock[1].color = nextBlock[2].color = nextBlock[3].color = &lblueBrickTex;
		break;
	case 4:
		nextBlock[0].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(21), 0.0f );
		nextBlock[1].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(19), 0.0f );
		nextBlock[2].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(20), 0.0f );
		nextBlock[3].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(19), 0.0f );
		nextBlock[0].color = nextBlock[1].color = nextBlock[2].color = nextBlock[3].color = &yellowBrickTex;
		break;
	case 5:
		nextBlock[0].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(21), 0.0f );
		nextBlock[1].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(19), 0.0f );
		nextBlock[2].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(20), 0.0f );
		nextBlock[3].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(19), 0.0f );
		nextBlock[0].color = nextBlock[1].color = nextBlock[2].color = nextBlock[3].color = &pinkBrickTex;
		break;
	case 6:
		nextBlock[0].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(21), 0.0f );
		nextBlock[1].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(20), 0.0f );
		nextBlock[2].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(20), 0.0f );
		nextBlock[3].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(19), 0.0f );
		nextBlock[0].color = nextBlock[1].color = nextBlock[2].color = nextBlock[3].color = &blueBrickTex;
		break;
	case 7:
		nextBlock[0].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(21), 0.0f );
		nextBlock[1].position = D3DXVECTOR3( PixelPositionX(5), PixelPositionY(20), 0.0f );
		nextBlock[2].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(20), 0.0f );
		nextBlock[3].position = D3DXVECTOR3( PixelPositionX(4), PixelPositionY(19), 0.0f );
		nextBlock[0].color = nextBlock[1].color = nextBlock[2].color = nextBlock[3].color = &greenBrickTex;	
		break;
	}
}

//---------------------------------------------------------------------------------------------------------------
// Name: DrawBackground()
// Desc: Draws the background, including the 'brick walls'
//---------------------------------------------------------------------------------------------------------------
void DrawBackground()
{
	bgPosition.y = 0.0f;
	bgPosition.x = 31.0f;
	for ( int i = 0 ; i < 40 ; i ++ )
	{
		sprite->Draw( bgTexture, NULL, NULL, &bgPosition , 0xffffffff );
		bgPosition.y += 16.0f;
	}
	bgPosition.y = 0.0f;
	bgPosition.x = 369.0f;
	for ( int i = 0 ; i < 40 ; i ++ )
	{
		sprite->Draw( bgTexture, NULL, NULL, &bgPosition , 0xffffffff );
		bgPosition.y += 16.0f;
	}

	// Draw the block that will come next - note: the text " Next Block " is drawn in DrawGameText()
	if ( gameStatus != paused )
	{
		for ( int i = 0 ; i < 4 ; i ++ )
			sprite->Draw( *nextBlock[i].color, NULL, NULL, &(nextBlock[i].position+D3DXVECTOR3( 295.0f, 160.0f, 0.0f ) )  , 0xffffffff );
	}
}

//---------------------------------------------------------------------------------------------------------------
// Name: DrawCurrentBlock()
// Desc: Draws the current block and moves it down at the speed setting of the parameter 
//---------------------------------------------------------------------------------------------------------------
void DrawCurrentBlock( int speed, int moveChance )
{
	for ( int a = 0; a < 4 ; a++ )
		sprite->Draw( *currentBlock[a].color , NULL, NULL,	&currentBlock[a].position, 0xffffffff );

	if ( timeGetTime()-begTime >= speed && moveChance == 0 ) //  Difference between begTime and current time
	{
		for ( int b = 0 ; b < 4 ; b++ )
			currentBlock[b].position += D3DXVECTOR3( 0.0f, 32.0f, 0.0f );	// Every (speed) milliseconds the block is moves down one unit of the grid
		begTime = timeGetTime(); // Assign begTime to the current, setting the difference to 0
	}
}

//--------------------------------------------------------------------------------------------------------------
// Name: RotateBlock()
// Desc: Rotates the current block if possible
//
//		 We accomplish the rotation using the formula:
//
//			x1 = x cos A + y sin A
//			y2 = y cos A - x sin A
//		
//		 Where x1 and y1 are the transformed x and y co-ordinates, x and y are the original co-ordinates, and A is the angle of rotation.
//
//		 Since we are rotating by 90 degrees... cos 90 = 0 and sin 90 = 1!
//
//		 This rotation is about the origin, so we first have to translate the block to make block 3 ( which in each block I have made the centre )
//		 the origin. To do this, all 4 positional vectors for the 4 blocks we take off the vector for block 3, making block 3 the center.
//
//--------------------------------------------------------------------------------------------------------------
void RotateBlock( BlockStruct activeBlock[4] )
{
	if ( activeBlock[0].color != &lblueBrickTex ) // The square block doesn't need to be rotated!
	{
		// Set the centre point of rotation for the object ( which I have set in all objects as the 3rd block, i.e. element[2] )
		D3DXVECTOR3 translate = activeBlock[2].position;

		// Assign the position vectors of the 4 blocks to 4 new vectors, so if we find our rotation is invalid, we still have the
		// original positions! 
		D3DXVECTOR3 validateRot[4] = { activeBlock[0].position, activeBlock[1].position, activeBlock[2].position, activeBlock[3].position };

		for ( int i = 0 ; i < 4 ; i++ )
			validateRot[i] -= translate; // Take off the 3rd block's vector 

		for ( int a = 0 ; a < 4 ; a++ )
		{
			float tempX = validateRot[a].x; 
			validateRot[a].x = 0 + validateRot[a].y; // x cos 90 = 0...
			validateRot[a].y = 0 - tempX;			 // y cos 90 = 0...
		}
		for ( int i = 0 ; i < 4 ; i++ )
			validateRot[i] += translate; // Translate back to original position
		
		// Loop to check if the rotation is valid
		bool isItValid = true;
		for ( int b = 0 ; b < 4 ; b++ )
		{
			if ( color[ GridPositionY(validateRot[b].y) ][ GridPositionX(validateRot[b].x) ] != NULL )
			{
				isItValid = false;
				break;
			}
			if ( validateRot[b].x < PixelPositionX(0) || validateRot[b].x > PixelPositionX(9) )
			{
				isItValid = false;
				break;
			}
		}

		// If it is assign the block the new co-ordinates. If isItValid is false the co-ordinates are not changed
		if ( isItValid != false )
		{
			for ( int c = 0 ; c < 4 ; c++ )
				activeBlock[c].position = validateRot[c];
		}
	}
}

//-------------------------------------------------------------------------------------------------------------
// Name: DrawGrid()
// Desc: Draws all blocks already on the grid
//-------------------------------------------------------------------------------------------------------------
void DrawGrid( LPDIRECT3DTEXTURE9* color[20][10] )
{
	for ( int y = 0 ; y < 20 ; y++ )
	{
		for ( int x = 0 ; x < 10 ; x++ )
		{
			if ( color[y][x] != NULL )
				sprite->Draw( *color[y][x], NULL, NULL,	&D3DXVECTOR3( PixelPositionX(x) , PixelPositionY(y) , 0.0f ), 0xffffffff );
		}
		
	}
}

//-------------------------------------------------------------------------------------------------------------
// Name: CheckForCollision()
// Desc: Checks if the block will hit a static block on the grid
//-------------------------------------------------------------------------------------------------------------
bool CheckForCollision( BlockStruct activeBlock[4] , LPDIRECT3DTEXTURE9* color[20][10] )
{
	bool stop = false;
	for ( int a = 0 ; a < 4 ; a++ )
	{
		if ( ! (( GridPositionY( activeBlock[a].position.y ) ) > 19) ) // Block has not come into the grid array area yet
		{
			if ( GridPositionY( activeBlock[a].position.y) == 0 ) // If the block reaches the bottom of the screen, stop it
			{
				stop = true;
				break;	
			}
			// If the block's position-1 ( y axis ) is taken by a static piece it can't move down, so stop it
			if ( color [ GridPositionY( activeBlock[a].position.y )-1 ][ GridPositionX( activeBlock[a].position.x) ] != NULL  )
			{
				stop = true;
				break;
			}
		}
	}

	return stop;
}

//-------------------------------------------------------------------------------------------------------------
// Name: UpdateGrid()
// Desc: Adds new entries to the grid
//-------------------------------------------------------------------------------------------------------------
void UpdateGrid( BlockStruct activeBlock[4], LPDIRECT3DTEXTURE9* color[20][10] )
{
		for( int i = 0 ; i < 4 ; i ++ )
		{
			if ( ! (GridPositionY( activeBlock[i].position.y ) > 19 ) )
				color[ GridPositionY( activeBlock[i].position.y ) ][ GridPositionX( activeBlock[i].position.x ) ] = currentBlock[i].color;
		}
	
}

//-------------------------------------------------------------------------------------------------------------
// Name: MoveBlockLeft()
// Desc: Move the block to the left by one unit, if possible
//-------------------------------------------------------------------------------------------------------------
void MoveBlockLeft( BlockStruct activeBlock[4], LPDIRECT3DTEXTURE9* color[20][10] )
{
	bool valid = true;
	for ( int a = 0 ; a < 4 ; a++ )
	{
		if ( activeBlock[a].position.x < PixelPositionX(1) )
		{
			valid = false;
			break;
		}
	}
	for ( int i = 0 ; i < 4 ; i++ )
	{
		if ( color[ GridPositionY(activeBlock[i].position.y) ][ GridPositionX(activeBlock[i].position.x)-1 ] != NULL )
		{
			valid = false;
			break;
		}
	}
	if ( valid != false )
	{
		for ( int i = 0 ; i < 4 ; i++ )
			activeBlock[i].position -= D3DXVECTOR3 ( 32.0f, 0.0f, 0.0f );
	}
}

//-------------------------------------------------------------------------------------------------------------
// Name: MoveBlockRight()
// Desc: Move the block to the right by one unit, if possible
//-------------------------------------------------------------------------------------------------------------
void MoveBlockRight( BlockStruct activeBlock[4], LPDIRECT3DTEXTURE9* color[20][10] )
{
	bool valid = true;
	for ( int a = 0 ; a < 4 ; a++ )
	{
		if ( activeBlock[a].position.x > PixelPositionX(8) )
		{
			valid = false;
			break;
		}
	}
	for ( int i = 0 ; i < 4 ; i++ )
	{
		if ( color[ GridPositionY(activeBlock[i].position.y) ][ GridPositionX(activeBlock[i].position.x)+1 ] != NULL )
		{
			valid = false;
			break;
		}
	}
	if ( valid != false )
	{
		for ( int i = 0 ; i < 4 ; i++ )
			activeBlock[i].position.x += 32.0f;
	}
}

//-------------------------------------------------------------------------------------------------------------
// Name: DropBlock()
// Desc: Drops the block straight down until it hits a static block
//-------------------------------------------------------------------------------------------------------------
void DropBlock( BlockStruct activeBlock[4] , LPDIRECT3DTEXTURE9* color[20][10] )
{
	numOfRowsDropped = 0;
	while ( CheckForCollision( activeBlock, color ) != true )
	{
		for ( int i = 0 ; i < 4 ; i++ )
			activeBlock[i].position.y += 32.0f;

		numOfRowsDropped++;
	}
}

//-------------------------------------------------------------------------------------------------------------
// Name: CheckForCompleteRow()
// Desc: Checks through all the rows checking to see if it is complete. If it is, get rid of it!
//-------------------------------------------------------------------------------------------------------------
void CheckForCompleteRow( LPDIRECT3DTEXTURE9* color[20][10] )
{	
	numOfRows = 0;
	for ( int y = 0 ; y < 20 ; y ++ )
	{
		bool isItComplete = true;
		for ( int x = 0 ; x < 10 ; x++ )
		{
			if ( color[ y ][ x ] == NULL )
			{ 
				isItComplete = false;
				break;
			}
		}

		if ( isItComplete != false )
		{
			numOfRows++;
			linesCleared++;
			Sleep( speed/2 ); // Give a little Time delay so it the player can actually see whats happening!
			// Shift all blocks above the complete row down
			int a = y;
			while ( a < 19 )
			{
				for ( int b = 0 ; b < 10 ; b++ )
				{
					color[ a ][ b ] = color[ a+1 ][ b ];
				}
				a++;
			}
			// Make the all values in the top row equal to NULL
			for ( int c = 0 ; c < 10 ; c++ )
				color[ 19 ][ c ] = NULL;

			y--; // All rows have been moved down, so the above row has to be checked again
		}

	}
}

//--------------------------------------------------------------------------------------------------------------
// Name: CheckForGameOver()
// Desc: Checks to see if the blocks have reached the top of the grid
//--------------------------------------------------------------------------------------------------------------
void CheckForGameOver( LPDIRECT3DTEXTURE9* color [20][10] )
{
		for ( int a = 0 ; a < 10 ; a++ )
		{
			if ( color[19][a] != NULL )
			{
				gameStatus = gameOver;
				break;
			}
		}
}

//--------------------------------------------------------------------------------------------------------------
// Name: UpdateScore()
// Desc: Adds however many points are due to the score
//--------------------------------------------------------------------------------------------------------------
void UpdateScore( int &score, int numOfCompleteRows, int &numOfRowsDropped )
{
	switch ( numOfCompleteRows ) // numOfRows is the number of complete rows found 
	{
	case 1:
		score += (level+1)*40;
		break;
	case 2:
		score += (level+1)*100;
		break;
	case 3:
		score += (level+1)*300;
		break;
	case 4:
		score += (level+1)*1200;
		break;
	}

	if ( numOfRowsDropped > 0 )
		score += (level+1)*numOfRowsDropped;

	numOfRowsDropped = 0;
}

//-------------------------------------------------------------------------------------------------------------
// Name: DrawGameText()
// Desc: Draws all the in game text. Score, level ... etc
//-------------------------------------------------------------------------------------------------------------
void DrawGameText( int score, int level )
{
	RECT scorePosition = { 400, 270, 640, 640 };
	RECT levelPosition = { 400, 460, 640, 640 };
	RECT nextBlockPosition = { 400, 20, 640, 640 };

	font->DrawText( NULL, "Next Block", -1, &nextBlockPosition, DT_LEFT, 0xffffffff );

	char scoreString[10];
	sprintf(scoreString,"%d",score);
	font->DrawText( NULL, "Score", -1, &scorePosition, DT_LEFT, 0xffffffff );
	scorePosition.top = 320;
	font->DrawText( NULL, (LPCSTR)scoreString, -1, &scorePosition, DT_CENTER, 0xffffffff );

	char levelString[10];
	sprintf(levelString,"%d",level);
	font->DrawText( NULL, "Level", -1, &levelPosition, DT_LEFT, 0xffffffff );
	levelPosition.top = 500;
	font->DrawText( NULL, (LPCSTR)levelString, -1, &levelPosition, DT_CENTER, 0xffffffff );
}

//-------------------------------------------------------------------------------------------------------------
// Name: MoveBlockDownOne()
// Desc: Move the block down by one unit, if possible
//-------------------------------------------------------------------------------------------------------------
void MoveBlockDownOne( BlockStruct activeBlock[4], LPDIRECT3DTEXTURE9* color[20][10] )
{
	bool valid = true;

	for ( int i = 0 ; i < 4 ; i++ )
	{
		if ( color[ GridPositionY(activeBlock[i].position.y)-1 ][ GridPositionX(activeBlock[i].position.x) ] != NULL )
		{
			valid = false;
			break;
		}
		if ( GridPositionY(activeBlock[i].position.y)-1 < 0 )
		{
			valid = false;
			break;
		}
	}

	
	if ( valid != false )
	{
		for ( int i = 0 ; i < 4 ; i++ )
			activeBlock[i].position += D3DXVECTOR3 ( 0.0f, 32.0f, 0.0f );
	}
}

//------------------------------------------------------------------------------------------------------------------------------------
// Name: UpdateHighScores()
// Desc: Checks whether you have reached the high score and adds it to it if you have
//------------------------------------------------------------------------------------------------------------------------------------
void UpdateHighScores( int &score )
{
	ifstream fin("highScores"); // Open highScores file
	if ( !fin )
	{
		MessageBox( NULL, " Can't find 'highScores' file, continuing happily without high scores", "Error", MB_OK );
		gameStatus = mainScreen;  // Change the screen that is displayed
		ResetVariables();
	}
	else
	{
		// Read in the values to the highScores array
		for ( int i = 0 ; i < 5 ; i++ )
		{
			fin >> highScore[i].name;
			fin >> highScore[i].score;
		}
		fin.close();
	
		if ( score > highScore[4].score )
		{
			 gameStatus = getNameForScoreboard;
			 for ( int a = 0 ; a < 10 ; a++ )
				 highScore[5].name[a] = '-';
		}
		else
			gameStatus = highScores;
	}		
}


//-------------------------------------------------------------------------------------------------------------
// Name: DrawGetNameForScoreScreen()
// Desc: 
//-------------------------------------------------------------------------------------------------------------
void DrawGetNameForScoreScreen( ScoreStruct highScore[5] )
{
	if ( g_pd3dDevice == NULL )
		return;

	RECT highScoreNotePosition = { 0, 102, 640, 400 };
	RECT highScorePositionName = { 140, 180, 640, 400 };
	RECT highScorePositionScore = { 450, 180, 640, 640 };
	RECT getNamePosition = { 150, 450, 640, 640 };


	// Clear the backbuffer
	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, backgroundColor[colorNum], 1.0f, 0 );

	if ( SUCCEEDED ( g_pd3dDevice->BeginScene() ) )
	{
		font->DrawText( NULL, "You've got a high score!", -1, &highScoreNotePosition, DT_CENTER, 0xff0fff0f );
		font->DrawText( NULL, "Enter your name: ", -1, &getNamePosition, DT_LEFT, 0xffffffff );
		getNamePosition.left += 100;
	
		for ( int i = 0 ; i < 5 ; i++ )
		{
	
			font->DrawText( NULL, (LPCSTR)highScore[i].name, -1, &highScorePositionName, DT_LEFT, 0xfff00f0f );
			char scoreString[10];
			sprintf(scoreString,"%d", highScore[i].score );
			font->DrawText( NULL, (LPCSTR)scoreString, -1, &highScorePositionScore, DT_LEFT, 0xfff00f0f );
			font->DrawText( NULL, (LPCSTR)highScore[5].name, -1, &getNamePosition, DT_CENTER, 0xffffffff );

			highScorePositionName.top += 40;
			highScorePositionScore.top += 40;
		}

		g_pd3dDevice->EndScene();
	}

	// Present the backbuffer contents to the display
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );

}

//------------------------------------------------------------------------------------------------------------
// Name: AddScoreToBoard()
// Desc: Adds the score to the leaderboard
//------------------------------------------------------------------------------------------------------------
void AddScoreToBoard( int score )
{
	int tempNum = highScore[5].name[0];
	highScore[5].name[0] = (char)(tempNum-32);
	for ( int b = 0 ; b < 10 ; b++ )
	{
		if (  highScore[5].name[b] == '-' )
			highScore[4].name[b] = ' ';
		else
			highScore[4].name[b] = highScore[5].name[b];		
	}
	highScore[4].score = score;

	//--------------------------------------------------
	// Bubble Sort
	ScoreStruct temp;
	for ( int pass = 1; pass < 5; pass++ )
	{
		for ( int a = 0; a < 5-pass ; a++ )
		{
			if ( highScore[a].score < highScore[a+1].score )
			{
				for ( int b = 0 ; b < 10 ; b++ )
					temp.name[b] = highScore[a].name[b];

				temp.score = highScore[a].score;
				for ( int b = 0 ; b < 10 ; b++ )
					highScore[a].name[b] = highScore[a+1].name[b];

				highScore[a].score = highScore[a+1].score;
				for ( int b = 0 ; b < 10 ; b++ )
					highScore[a+1].name[b] = temp.name[b];

				highScore[a+1].score = temp.score;
			}
		}
	}
	//-------------------------------------------------

	ofstream fout("highScores");	// Open highScores for writing
	for ( int a = 0 ; a < 5 ; a++ )
	{
		fout << highScore[a].name << " ";
		fout << highScore[a].score << endl;
	}
	fout.close();
}
