//---------------------------------------------------------------------------------------------------------------------------------
// Name: DrawLevelUpScreen()
// Desc: Draws only the background and game text - the blocks have been cleared - and also displays a level up message
//---------------------------------------------------------------------------------------------------------------------------------
void DrawLevelUpScreen()
{
	if ( g_pd3dDevice == NULL )		return;

	RECT pausedTextPosition = { 48, 300, 360, 640 };

	// Clear the backbuffer
	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, backgroundColor[colorNum], 1.0f, 0 );

	if ( SUCCEEDED ( g_pd3dDevice->BeginScene() ) )
	{
		sprite->Begin( D3DXSPRITE_ALPHABLEND );
			
			DrawBackground(); // Wonder what this function does? :D
			DrawGameText( score, level ); // Draw the score, level etc.

			font->DrawText( NULL, "Level Up!", -1, &pausedTextPosition, DT_CENTER, 0xfffff00f );
			
		sprite->End();

		g_pd3dDevice->EndScene();
	}

	// Present the backbuffer contents to the display
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}

//-----------------------------------------------------
// Name: Render()
// Desc: Renders the scene
//-----------------------------------------------------

void Render()
{
	if ( g_pd3dDevice == NULL )
		return;

	// Clear the backbuffer
	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, backgroundColor[colorNum], 1.0f, 0 );

	if ( SUCCEEDED ( g_pd3dDevice->BeginScene() ) )
	{
		sprite->Begin( D3DXSPRITE_ALPHABLEND );

				DrawBackground(); // Wonder what this function does? :D
				DrawGameText( score, level );
				DrawGrid( color ); // Draw all the static bricks on the grid
				if ( CheckForCollision( currentBlock, color ) == true ) // The block can't go down any further
				{
					//CheckForGameOver( currentBlock, color );
					moveChance++; // You have the time it takes for moveChance to get to (speed/20) to move the block once it has come into contact with a static one
					DrawCurrentBlock( speed, 1 ); // Draw the block, parameter 2 set to anything greater than 0 so the block isn't moved down
					
					if ( moveChance >= speed/20 ) // Time up, add block to grid
					{
						UpdateGrid( currentBlock, color ); // Add the blocks's bricks to the grid 
						CheckForGameOver( color );
						if ( gameStatus != gameOver )
						{
							CheckForCompleteRow( color );
							UpdateScore( score, numOfRows, numOfRowsDropped );
							UpdateLevel( linesCleared, levelUp );
							CreateNextBlock( nextBlock ); // Create a new block
							moveChance = 0; // Reset 
						}
					}	
				}
				else 
					DrawCurrentBlock( speed, 0 ); // Draw the block and move it down. Parameter 2 is set to 0 since anything > 0 wont let it move down
		
		sprite->End();

		g_pd3dDevice->EndScene();
	}

	// Present the backbuffer contents to the display
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );

}

//---------------------------------------------------------------------------------------------------------------------------------
// Name: DrawPausedScreen()
// Desc: Draws only the background and game text so you cannot see the blocks when the game is paused, that would be cheating!
//---------------------------------------------------------------------------------------------------------------------------------
void DrawPausedScreen()
{
	if ( g_pd3dDevice == NULL )		return;

	RECT pausedTextPosition = { 48, 300, 360, 640 };

	// Clear the backbuffer
	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, backgroundColor[colorNum], 1.0f, 0 );

	if ( SUCCEEDED ( g_pd3dDevice->BeginScene() ) )
	{			
		DrawBackground(); // Wonder what this function does? :D
		DrawGameText( score, level ); // Draw the score, level etc.

		font->DrawText( NULL, "Paused", -1, &pausedTextPosition, DT_CENTER, 0xfffff00f );
		pausedTextPosition.top += 35;
		font->DrawText( NULL, "Press 'p' to continue", -1, &pausedTextPosition, DT_CENTER, 0xfffff00f );
		
		g_pd3dDevice->EndScene();
	}

	// Present the backbuffer contents to the display
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}

//---------------------------------------------------------------------------------------------------------------------------------
// Name: DrawGameOverScreen()
// Desc: Draws the game over screen
//---------------------------------------------------------------------------------------------------------------------------------
void DrawGameOverScreen()
{
	if ( g_pd3dDevice == NULL ) // Check the device
		return;
	D3DXVECTOR3 gameOverSpritePosition ( 200.0f, 80.0f, 0.0f );
	RECT endScorePosition = { 250, 270, 640, 640 };


	// Clear the backbuffer
	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, D3DCOLOR_XRGB( 0,0, 0 ), 1.0f, 0 ); // Clear screen to black

	if ( SUCCEEDED ( g_pd3dDevice->BeginScene() ) )
	{
		sprite->Begin( D3DXSPRITE_ALPHABLEND );
			
			sprite->Draw( gameOverScreenTex, NULL, NULL, &gameOverSpritePosition , 0xffffffff );
			
		sprite->End();

		char scoreString[10];
		sprintf(scoreString,"%d",score);
		font->DrawText( NULL, "Score:", -1, &endScorePosition, DT_LEFT, 0xfffff00f );
		endScorePosition.left += 100;
		font->DrawText( NULL, (LPCSTR)scoreString, -1, &endScorePosition, DT_LEFT, 0xfffff00f );
		endScorePosition.left = 0;
		endScorePosition.top +=200;

		g_pd3dDevice->EndScene();
	}

	// Present the backbuffer contents to the display
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}

//-------------------------------------------------------------------------------------------------------------
// Name: DrawHighScoreScreen()
// Desc: 
//-------------------------------------------------------------------------------------------------------------
void DrawHighScoreScreen( ScoreStruct highScore[4] )
{
	if ( g_pd3dDevice == NULL )		return;

	RECT highScorePositionName = { 140, 180, 640, 400 };
	RECT highScorePositionScore = { 450, 180, 640, 640 };
	RECT getNamePosition = { 0, 100, 640, 640 };


	// Clear the backbuffer
	g_pd3dDevice->Clear( 0, NULL, D3DCLEAR_TARGET, backgroundColor[colorNum], 1.0f, 0 );

	if ( SUCCEEDED ( g_pd3dDevice->BeginScene() ) )
	{
		font->DrawText( NULL, "HIGH SCORES", -1, &getNamePosition, DT_CENTER, 0xff0fff0f );
		getNamePosition.top += 500;
		font->DrawText( NULL, "press s for main menu", -1, &getNamePosition, DT_CENTER, 0xff0fff0f );
		
		for ( int i = 0 ; i < 5 ; i++ )
		{
	
			font->DrawText( NULL, (LPCSTR)highScore[i].name, -1, &highScorePositionName, DT_LEFT, 0xfff00f0f );
			char scoreString[10];
			sprintf(scoreString,"%d", highScore[i].score );
			font->DrawText( NULL, (LPCSTR)scoreString, -1, &highScorePositionScore, DT_LEFT, 0xfff00f0f );

			highScorePositionName.top += 40;
			highScorePositionScore.top += 40;
		}

		g_pd3dDevice->EndScene();
	}

	// Present the backbuffer contents to the display
	g_pd3dDevice->Present( NULL, NULL, NULL, NULL );
}


	