#include "DBloxFunctions.h"
#include "RenderFunctions.h"
#include "Menu.h"

#include "resource.h"

//------------------------------------------------------
// Name: InitD3D()
// Desc: Initiates Direct3D
//------------------------------------------------------
HRESULT InitD3D( HWND hWnd )
{
	if ( NULL == ( g_pD3D = Direct3DCreate9 ( D3D_SDK_VERSION )))
		return E_FAIL;

	D3DPRESENT_PARAMETERS d3dpp;
	ZeroMemory( &d3dpp, sizeof(d3dpp) );
	d3dpp.Windowed = TRUE;
	d3dpp.SwapEffect = D3DSWAPEFFECT_DISCARD;
	d3dpp.BackBufferFormat = D3DFMT_A8R8G8B8;
	d3dpp.BackBufferWidth = 640;				// Size of the game
	d3dpp.BackBufferHeight = 640;

	// Create the device
	if( FAILED ( g_pD3D->CreateDevice ( D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, hWnd, D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &g_pd3dDevice ) ) )
		return E_FAIL;

	// Create the textures from the *.bmp's
	if ( FAILED ( CreateTextures() ) )
		return E_FAIL;

	// Create a sprite used to draw the textures to the screen
	if ( FAILED ( D3DXCreateSprite( g_pd3dDevice, &sprite ) ) )
		return E_FAIL;

	if ( FAILED ( D3DXCreateFont( g_pd3dDevice, 30, 0, 0, 0, 0, DEFAULT_CHARSET, OUT_DEFAULT_PRECIS, DEFAULT_QUALITY, 
								  DEFAULT_PITCH | FF_DONTCARE, "Arial", &font ) ) )
	{
		return E_FAIL;
	}

	CreateNextBlock( currentBlock ); // Create first block
	CreateNextBlock( nextBlock ); // Create next block, to be shown in the top right corner
	
	return S_OK;

}

//-----------------------------------------------------
// Name: Cleanup()
// Desc: Releases initialized variables
//-----------------------------------------------------
void Cleanup()
{
	if ( font != NULL )
		font->Release();

	for ( int y = 0 ; y < 20 ; y++ )
	{
		for ( int x = 0 ; x < 10 ; x++ )
		{
			color[y][x] = NULL;
		}
	}

	for ( int a = 0 ; a < 4 ; a++ )
	{
		nextBlock[a].color = NULL;
	}

	if ( helpScreenTex != NULL )
		helpScreenTex->Release();

	if ( mainScreenTex != NULL )
		mainScreenTex->Release();

	if ( gameOverScreenTex != NULL )
		gameOverScreenTex->Release();

	if ( bgTexture != NULL )
		bgTexture->Release();

	if ( blueBrickTex != NULL )
		blueBrickTex->Release();

	if ( redBrickTex != NULL )
		redBrickTex->Release();

	if ( lblueBrickTex != NULL )
		lblueBrickTex->Release();

	if ( greenBrickTex != NULL )
		greenBrickTex->Release();

	if ( yellowBrickTex != NULL )
		yellowBrickTex->Release();

	if ( pinkBrickTex != NULL )
		pinkBrickTex->Release();

	if ( greyBrickTex != NULL )
		greyBrickTex->Release();

	if ( sprite != NULL )
		sprite->Release();

	if ( g_pd3dDevice != NULL )
		g_pd3dDevice->Release();
	
	if ( g_pD3D != NULL )
		g_pD3D->Release();
}


//----------------------------------------------------
// Name: MsgProc()
// The applications message handler
//----------------------------------------------------
LRESULT WINAPI MsgProc( HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
	
	if ( gameStatus == getNameForScoreboard ) // Switch between message proc modes - when on getNameForScoreboard screen all letters produce letters and 'n' doesn't start a new game
	{
		switch( msg )
		{
		case WM_DESTROY:
			Cleanup();
			PostQuitMessage( 0 );
			return 0;
		case WM_KEYDOWN:
			if ( nameArrayPos > 9 && wParam != 8 && wParam != 13 )
				break;
			switch ( wParam )
			{
			case 8: // Backspace
				if ( nameArrayPos > 0 )
				{					
					highScore[5].name[nameArrayPos-1] = '-';
					nameArrayPos--;
				}
				break;
			case 13: // Enter
				AddScoreToBoard( score );
				gameStatus = highScores;
				break;
			case 65:
				highScore[5].name[nameArrayPos] = 'a';
				nameArrayPos++;
				break;
			case 66:
				highScore[5].name[nameArrayPos] = 'b';
				nameArrayPos++;
				break;
			case 67:
				highScore[5].name[nameArrayPos] = 'c';
				nameArrayPos++;
				break;
			case 68:
				highScore[5].name[nameArrayPos] = 'd';
				nameArrayPos++;
				break;
			case 69:
				highScore[5].name[nameArrayPos] = 'e';
				nameArrayPos++;
				break;
			case 70:
				highScore[5].name[nameArrayPos] = 'f';
				nameArrayPos++;
				break;
			case 71:
				highScore[5].name[nameArrayPos] = 'g';
				nameArrayPos++;
				break;
			case 72:
				highScore[5].name[nameArrayPos] = 'h';
				nameArrayPos++;
				break;
			case 73:
				highScore[5].name[nameArrayPos] = 'i';
				nameArrayPos++;
				break;
			case 74:
				highScore[5].name[nameArrayPos] = 'j';
				nameArrayPos++;
				break;
			case 75:
				highScore[5].name[nameArrayPos] = 'k';
				nameArrayPos++;
				break;
			case 76:
				highScore[5].name[nameArrayPos] = 'l';
				nameArrayPos++;
				break;
			case 77:
				highScore[5].name[nameArrayPos] = 'm';
				nameArrayPos++;
				break;
			case 78:
				highScore[5].name[nameArrayPos] = 'n';
				nameArrayPos++;
				break;
			case 79:
				highScore[5].name[nameArrayPos] = 'o';
				nameArrayPos++;
				break;
			case 80:
				highScore[5].name[nameArrayPos] = 'p';
				nameArrayPos++;
				break;
			case 81:
				highScore[5].name[nameArrayPos] = 'q';
				nameArrayPos++;
				break;
			case 82:
				highScore[5].name[nameArrayPos] = 'r';
				nameArrayPos++;
				break;
			case 83:
				highScore[5].name[nameArrayPos] = 's';
				nameArrayPos++;
				break;
			case 84:
				highScore[5].name[nameArrayPos] = 't';
				nameArrayPos++;
				break;
			case 85:
				highScore[5].name[nameArrayPos] = 'u';
				nameArrayPos++;
				break;
			case 86:
				highScore[5].name[nameArrayPos] = 'v';
				nameArrayPos++;
				break;
			case 87:
				highScore[5].name[nameArrayPos] = 'w';
				nameArrayPos++;
				break;
			case 88:
				highScore[5].name[nameArrayPos] = 'x';
				nameArrayPos++;
				break;
			case 89:
				highScore[5].name[nameArrayPos] = 'y';
				nameArrayPos++;
				break;
			case 90:
				highScore[5].name[nameArrayPos] = 'z';
				nameArrayPos++;
				break;
			}
		}
	}
	
	else
	{

	switch ( msg )
	{
	case WM_DESTROY:
		Cleanup();
		PostQuitMessage( 0 );
		return 0;
	
	case WM_KEYDOWN:
		switch ( wParam )
		{
		case 38: // Up Arrow
			RotateBlock( currentBlock );
			break;
		case 37: // Left Arrow
			MoveBlockLeft( currentBlock, color );
			break;
		case 39: // Right Arrow
			MoveBlockRight( currentBlock, color );
			break;
		case 32: // Space
			if ( gameStatus == inGame )
			{
				DropBlock( currentBlock, color );
				moveChance = 50; // No chance to move it when it hits a static block
			}
			break;
		case 40: // Down Arrow
			MoveBlockDownOne( currentBlock, color );
			begTime = timeGetTime(); // Reset begTime so the block doesn't jump down after we press down
			break;
		case 67: // Letter 'c'
			if ( colorNum >= 7 )
				colorNum = 0;
			else
				colorNum++;
			break;
		case 72: // Letter 'h'
			if ( gameStatus == mainScreen || gameStatus == helpScreen )
			{
				if ( gameStatus == helpScreen ) // Switch between help screen and main screen
					gameStatus = mainScreen;
				else 
					gameStatus = helpScreen;
			}
			break;
		case 78: // Letter 'n'
			if ( gameStatus == mainScreen )
			{
				ResetVariables();
				gameStatus = inGame;
			}
			break;
		case 80: // Letter 'p'
			if ( gameStatus == inGame || gameStatus == paused )
			{
				if ( gameStatus == paused ) // Switch between paused screen and inGame screen
					gameStatus = inGame;					
				else 
					gameStatus = paused;
			}
			break;
		case 83: // Letter 's'
			if ( gameStatus == mainScreen )
			{
				gameStatus = highScores;
				UpdateHighScores( score );
			}
			else if ( gameStatus == highScores )
				gameStatus = mainScreen;
			break;
		}
	}
	}

	DefWindowProc( hWnd, msg, wParam, lParam );
}

//---------------------------------------------------
// Name: WinMain()
// Desc: The apps main entry point
//---------------------------------------------------
INT WINAPI WinMain( HINSTANCE hInstance, HINSTANCE, LPSTR, INT )
{
	// Create a window class
	WNDCLASSEX wc = { sizeof(WNDCLASSEX), CS_CLASSDC, MsgProc, 0L, 0L, GetModuleHandle( NULL ), NULL, NULL, NULL, NULL, "DBloxGame", NULL };

	RegisterClassEx (&wc);
	
	HWND hWnd = CreateWindow( "DBloxGame", "DBlox", WS_OVERLAPPEDWINDOW, 100, 80, 640, 640, NULL, NULL, wc.hInstance, NULL );

	// Initialize Direct3D
	if ( SUCCEEDED ( InitD3D ( hWnd ) ) )
	{
			ShowWindow( hWnd, SW_SHOWDEFAULT );
			UpdateWindow( hWnd );
	
			MSG msg;
			ZeroMemory( &msg, sizeof(msg) );
	
			while ( msg.message != WM_QUIT )
			{
				if ( PeekMessage( &msg, NULL, 0U, 0U, PM_REMOVE ) )
				{
					TranslateMessage( &msg );
					DispatchMessage( &msg );
				}
				else if ( gameStatus == gameOver ) 
				{
					DrawGameOverScreen();
					Sleep( 1500 );
					UpdateHighScores( score );
				}
				
				else if ( gameStatus == mainScreen )
					DrawMenu();
				
				else if ( gameStatus == helpScreen ) 
					DrawHelpScreen();
			
				else if ( gameStatus == paused )
					DrawPausedScreen();

				else if ( gameStatus == highScores )
					DrawHighScoreScreen( highScore );

				else if ( gameStatus == getNameForScoreboard )
					DrawGetNameForScoreScreen( highScore );
				
				else if ( levelUp == true ) // If levelUp is true display the level up screen
				{
					DrawLevelUpScreen();
					Sleep( 800 );
					levelUp = false; // Reset
				}
				else
					Render(); // If none of the above are true then  gameStatus == inGame so draw the inGame screen
			}
		
		
	}

	UnregisterClass( "DBloxGame", wc.hInstance );
	return 0;
}




